function [g,info] = simplesaturationgain(G,SatDim,Qconst)
% function [g,info] = simplesaturationgain(G,SatDim,Qconst)
%
% This function computes a bound on gain of Fu(G,Delta) where G is an
% LPV system and Delta a saturation given by a sector bound.
% The code makes the following
% assumptions to study the use of parameter-varying IQCs:
%   1) G depends on a single parameter p
%   2) There are no rate bounds on the parameter p
%   3) The delayed signal can have multiple dimensions, i.e. Delta
%       denotes a MIMO operator with I/O dimension specified by SatDim.
%   4) The condition uses the following parameter-varying IQC for Delta:
%         [0 Q(p(t)); Q(p(t)) -2Q(p(t))]  
%      where Q(p) is diagonally dominant for all parameters p.
%      This corresponds to the saturation belonging to the sector [0,1]
%
% Inputs
%  G: An (nv+ne)-by-(nw+nd)-by-nmod array of state-space models that 
%      describe the dynamics of the LPV system G on a parameter grid of 
%      nmod points.  nv=nw are the dimensions of the delay (=DelayDim)
%      and nd/ne are the dimensions of the disturbance/error signals
%      of the resulting interconnection Fu(G,Stau*I).
%  SatDim: Integer describing the dimension of the saturation.
%  Qconst: Flag to switch between the use of a constant IQC (Qconst==true)
%      and a parameter varying IQC (Qconst=false).  This input is optional 
%      and default is Qconst==true.
%
% Outputs
%  g: Bound on the gain of Fu(G,Delta)
%  info:  Structure with information regarding solution of dissipation
%         inequality. Structure includes the fields:
%    -P:  Symmetric matrix used to construct the storage function.
%    -Q:  Matrices for the parameter-varyign IQC.  info.Q is (constant) 
%         matrix of SatDim-by-SatDim if Qconst=true. If Qconst=false 
%         then info.Q is an SatDim-by-SatDim-by-nmod array of matrices
%         specifying the parameter-dependent IQC.
%

% 3/14/2014 HP Coding for specific parameter-varying IQC example

%% Check inputs
if nargin==2
    Qconst=true;
end

%% Dimensions
szG = size(G);
if numel(szG) == 2
    szG = [szG 1];
end
nmod = szG(3); % # of models in the G array.

nv = SatDim;  % Size of input to Saturation
nw = nv;        % Size of output from Saturation

nd = szG(2)-nw;   % # of disturbances
ne = szG(1)-nv;   % # of errors

if Qconst
    % Use only a single (constant) Q
    nq = 1;
else
    % Use a parameter varying Q specified by a value on each grid point.
    nq = nmod;
end


%%
% The feedback interconnection is described by:
%    [v;e] = G*[w;d]    and w = Delta(v)
% Construct extended system Pext that has the following I/O:
%    [v;w;e]=Pext*[w;d]
% The [v;w] channels will be constrained by the IQC.
[A,B,C,D]=ssdata(G);
nx = size(A,1);
Cext = zeros(nv+nw+ne,nx,nmod);
Dext = zeros(nv+nw+ne,nw+nd,nmod);
for i=1:nmod
    Cext(:,:,i) = [C(1:nv,:,i); zeros(nw,nx); C(nv+1:end,:,i)];
    Dext(:,:,i) = [D(1:nv,:,i); eye(nw) zeros(nw,nd); D(nv+1:end,:,i)];
end

nz = nw+nv;
Cz = Cext(1:nz,:,:);
Dz = Dext(1:nz,:,:);

Ce = Cext(nz+1:end,:,:);
De = Dext(nz+1:end,:,:);

%%
% Initialize LMI and define variables
setlmis([])
P = lmivar(1,[nx 1]);
if Qconst
    [Q,~,tmp]=lmivar(1,[nv 1]);
    Qdec = cell(nmod,1);
    for i=1:nmod
        Qdec{i} = tmp;
    end
else
    Q = zeros(nq,1);
    Qdec = cell(nq,1);
    for i=1:nq
        [Q(i),~,Qdec{i}]=lmivar(1,[nv 1]);
    end
end
[gsq,ndec] = lmivar(1,[1 1]);
gsqI = lmivar(3,blkdiag(zeros(nx+nw),ndec*eye(nd)));

%%
% Specify LMIs

% LMI for Dissipation Inequality
cnt = 1;
for k = 1:nmod
    Ak = A(:,:,k);
    Bk = B(:,:,k);
    Cek = Ce(:,:,k);
    Dek = De(:,:,k);
    Czk = Cz(:,:,k);
    Dzk = Dz(:,:,k);
    
    % Terms involving plant and error signals
    lmiterm([cnt 1 1 P],[eye(nx); zeros(nw+nd,nx)],[Ak Bk],'s');
    lmiterm([-cnt 1 1 gsqI],1,1);
    lmiterm([cnt 1 1 0],[Cek Dek]'*[Cek Dek]);
        
    % Terms involving IQC for Delta
    Rfac = [Czk Dzk];
    Mk1 = lmivar(3, [zeros(nv), Qdec{k}; Qdec{k}, zeros(nv)]);
    Mk2 = lmivar(3, blkdiag(zeros(nv), Qdec{k}));
    lmiterm([cnt 1 1 Mk1],Rfac',Rfac);
    lmiterm([cnt 1 1 Mk2],-2*Rfac',Rfac);
    cnt = cnt +1;
end

% LMI to enforce P>=0
lmiterm([-cnt 1 1 P],1,1);
cnt  = cnt+1;

% LMI to enforce scalings to be diagonally dominant
for j=1:nq
    for k = 1:nv
        tmp = lmivar(3,Qdec{j}(k,k));
        lmiterm([-cnt 1 1 tmp],1,1);
        for i = 1:nw
            cnt1 = 1;
            if i ~= k
                tmp = lmivar(3,Qdec{j}(k,i));
                lmiterm([-cnt 1 1 tmp],1,1);
                lmiterm([cnt+cnt1 1 1 tmp],1,1);
                cnt1 = cnt1 + 1;
            end
        end
        cnt = cnt + cnt1 + 1;
    end
end

%% Solve SDP: min gamsq subject to LMI constraints
lmisys = getlmis;
c = zeros(ndec,1);
c(end) = 1;

opt = [0 0 0 0 1];
[copt,xopt] = mincx(lmisys,c,opt);

%% Store solution
g = inf;
info.P = [];
info.Q = [];
if ~isempty(xopt)
    g = sqrt(copt);
    info.P = dec2mat(lmisys,xopt,P);
    Qopt = zeros(nv,nv,nq);
    for j=1:nq
        Qopt(:,:,j) = dec2mat(lmisys,xopt,Q(j));
    end
    info.Q = Qopt;
end
